@@ -401,7 +401,7 @@ def get_module_part(dotted_name, context_file=None):
     return dotted_name
 
 
-def get_module_files(src_directory, blacklist, list_all=False):
+def get_module_files(src_directory, blacklist):
     """given a package directory return a list of all available python
     module's files in the package and its subpackages
 
@@ -413,10 +413,6 @@ def get_module_files(src_directory, blacklist, list_all=False):
     :param blacklist: iterable
       list of files or directories to ignore.
 
-    :type list_all: bool
-    :param list_all:
-        get files from all paths, including ones without __init__.py
-
     :rtype: list
     :return:
       the list of all available python module's files in the package and
@@ -426,7 +422,7 @@ def get_module_files(src_directory, blacklist, list_all=False):
     for directory, dirnames, filenames in os.walk(src_directory):
         _handle_blacklist(blacklist, dirnames, filenames)
         # check for __init__.py
-        if not list_all and not '__init__.py' in filenames:
+        if not '__init__.py' in filenames:
             dirnames[:] = ()
             continue
         for filename in filenames:
