@@ -23,8 +23,6 @@ import collections
 import operator
 import sys
 
-import six
-
 from astroid import arguments
 from astroid import bases
 from astroid import context as contextmod
@@ -33,18 +31,9 @@ from astroid import node_classes
 from astroid import nodes
 from astroid import util
 
-
-def _reflected_name(name):
-    return "__r" + name[2:]
-
-def _augmented_name(name):
-    return "__i" + name[2:]
-
-
-_CONTEXTLIB_MGR = 'contextlib.contextmanager'
 BIN_OP_METHOD = {'+':  '__add__',
                  '-':  '__sub__',
-                 '/':  '__div__' if six.PY2 else '__truediv__',
+                 '/':  '__div__',
                  '//': '__floordiv__',
                  '*':  '__mul__',
                  '**': '__pow__',
@@ -57,40 +46,41 @@ BIN_OP_METHOD = {'+':  '__add__',
                  '@': '__matmul__'
                 }
 
-REFLECTED_BIN_OP_METHOD = {
-    key: _reflected_name(value)
-    for (key, value) in BIN_OP_METHOD.items()
-}
-AUGMENTED_OP_METHOD = {
-    key + "=": _augmented_name(value)
-    for (key, value) in BIN_OP_METHOD.items()
-}
-
 UNARY_OP_METHOD = {'+': '__pos__',
                    '-': '__neg__',
                    '~': '__invert__',
                    'not': None, # XXX not '__nonzero__'
                   }
-_UNARY_OPERATORS = {
-    '+': operator.pos,
-    '-': operator.neg,
-    '~': operator.invert,
-    'not': operator.not_,
-}
+
+# unary operations ############################################################
+
+def tl_infer_unary_op(self, operator):
+    if operator == 'not':
+        return node_classes.const_factory(not bool(self.elts))
+    raise TypeError() # XXX log unsupported operation
+nodes.Tuple.infer_unary_op = tl_infer_unary_op
+nodes.List.infer_unary_op = tl_infer_unary_op
 
 
-def _infer_unary_op(obj, op):
-    func = _UNARY_OPERATORS[op]
-    value = func(obj)
-    return nodes.const_factory(value)
+def dict_infer_unary_op(self, operator):
+    if operator == 'not':
+        return node_classes.const_factory(not bool(self.items))
+    raise TypeError() # XXX log unsupported operation
+nodes.Dict.infer_unary_op = dict_infer_unary_op
 
-nodes.Tuple.infer_unary_op = lambda self, op: _infer_unary_op(tuple(self.elts), op)
-nodes.List.infer_unary_op = lambda self, op: _infer_unary_op(self.elts, op)
-nodes.Set.infer_unary_op = lambda self, op: _infer_unary_op(set(self.elts), op)
-nodes.Const.infer_unary_op = lambda self, op: _infer_unary_op(self.value, op)
-nodes.Dict.infer_unary_op = lambda self, op: _infer_unary_op(dict(self.items), op)
 
-# Binary operations
+def const_infer_unary_op(self, operator):
+    if operator == 'not':
+        return node_classes.const_factory(not self.value)
+    # XXX log potentially raised TypeError
+    elif operator == '+':
+        return node_classes.const_factory(+self.value)
+    else: # operator == '-':
+        return node_classes.const_factory(-self.value)
+nodes.Const.infer_unary_op = const_infer_unary_op
+
+
+# binary operations ###########################################################
 
 BIN_OP_IMPL = {'+':  lambda a, b: a + b,
                '-':  lambda a, b: a - b,
@@ -105,81 +95,107 @@ BIN_OP_IMPL = {'+':  lambda a, b: a + b,
                '<<': lambda a, b: a << b,
                '>>': lambda a, b: a >> b,
               }
+
 if sys.version_info >= (3, 5):
     # MatMult is available since Python 3.5+.
     BIN_OP_IMPL['@'] = operator.matmul
 
-for _KEY, _IMPL in list(BIN_OP_IMPL.items()):
-    BIN_OP_IMPL[_KEY + '='] = _IMPL
+for key, impl in list(BIN_OP_IMPL.items()):
+    BIN_OP_IMPL[key+'='] = impl
 
-
-@bases.yes_if_nothing_inferred
-def const_infer_binary_op(self, operator, other, context, _):
-    not_implemented = nodes.Const(NotImplemented)
-    if isinstance(other, nodes.Const):
-        try:
-            impl = BIN_OP_IMPL[operator]
+def const_infer_binary_op(self, operator, other, context):
+    for other in other.infer(context):
+        if isinstance(other, nodes.Const):
             try:
-                yield nodes.const_factory(impl(self.value, other.value))
+                impl = BIN_OP_IMPL[operator]
+
+                try:
+                    yield node_classes.const_factory(impl(self.value, other.value))
+                except Exception:
+                    # ArithmeticError is not enough: float >> float is a TypeError
+                    # TODO : let pylint know about the problem
+                    pass
             except TypeError:
-                # ArithmeticError is not enough: float >> float is a TypeError
-                yield not_implemented
-            except Exception: # pylint: disable=broad-except
+                # XXX log TypeError
+                continue
+        elif other is util.YES:
+            yield other
+        else:
+            try:
+                for val in other.infer_binary_op(operator, self, context):
+                    yield val
+            except AttributeError:
                 yield util.YES
-        except TypeError:
-            yield not_implemented
-    elif isinstance(self.value, six.string_types) and operator == '%':
-        # TODO(cpopa): implement string interpolation later on.
-        yield util.YES
-    else:
-        yield not_implemented
+nodes.Const.infer_binary_op = bases.yes_if_nothing_inferred(const_infer_binary_op)
 
-nodes.Const.infer_binary_op = const_infer_binary_op
 
 
 def _multiply_seq_by_int(self, other, context):
     node = self.__class__()
-    elts = [n for elt in self.elts for n in elt.infer(context)
-            if not n is util.YES] * other.value
-    node.elts = elts
+    elts = []
+    for elt in self.elts:
+        infered = util.safe_infer(elt, context)
+        if infered is None:
+            infered = util.YES
+        elts.append(infered)
+    node.elts = elts * other.value
     return node
 
 
-@bases.yes_if_nothing_inferred
-def tl_infer_binary_op(self, operator, other, context, method):
-    not_implemented = nodes.Const(NotImplemented)
-    if isinstance(other, self.__class__) and operator == '+':
-        node = self.__class__()
-        elts = [n for elt in self.elts for n in elt.infer(context)
-                if not n is util.YES]
-        elts += [n for elt in other.elts for n in elt.infer(context)
-                 if not n is util.YES]
-        node.elts = elts
-        yield node
-    elif isinstance(other, nodes.Const) and operator == '*':
-        if not isinstance(other.value, int):
-            yield not_implemented
-            return
-        yield _multiply_seq_by_int(self, other, context)
-    elif isinstance(other, bases.Instance) and operator == '*':
-        # Verify if the instance supports __index__.
-        as_index = class_as_index(other, context)
-        if not as_index:
-            yield util.YES
+def _filter_uninferable_nodes(elts, context):
+    for elt in elts:
+        if elt is util.YES:
+            yield elt
         else:
-            yield _multiply_seq_by_int(self, as_index, context)
+            for inferred in elt.infer(context):
+                yield inferred
+
+
+def tl_infer_binary_op(self, operator, other, context):
+    for other in other.infer(context):
+        if isinstance(other, self.__class__) and operator == '+':
+            node = self.__class__()
+            elts = list(_filter_uninferable_nodes(self.elts, context))
+            elts += list(_filter_uninferable_nodes(other.elts, context))
+            node.elts = elts
+            yield node
+        elif isinstance(other, nodes.Const) and operator == '*':
+            if not isinstance(other.value, int):
+                yield util.YES
+                continue
+            yield _multiply_seq_by_int(self, other, context)
+        elif isinstance(other, bases.Instance) and not isinstance(other, nodes.Const):
+            yield util.YES
+    # XXX else log TypeError
+nodes.Tuple.infer_binary_op = bases.yes_if_nothing_inferred(tl_infer_binary_op)
+nodes.List.infer_binary_op = bases.yes_if_nothing_inferred(tl_infer_binary_op)
+
+
+def dict_infer_binary_op(self, operator, other, context):
+    for other in other.infer(context):
+        if isinstance(other, bases.Instance) and isinstance(other._proxied, nodes.ClassDef):
+            yield util.YES
+        # XXX else log TypeError
+nodes.Dict.infer_binary_op = bases.yes_if_nothing_inferred(dict_infer_binary_op)
+
+def instance_infer_binary_op(self, operator, other, context):
+    try:
+        methods = self.getattr(BIN_OP_METHOD[operator])
+    except (exceptions.NotFoundError, KeyError):
+        # Unknown operator
+        yield util.YES
     else:
-        yield not_implemented
+        for method in methods:
+            if not isinstance(method, nodes.FunctionDef):
+                continue
+            for result in method.infer_call_result(self, context):
+                if result is not util.YES:
+                    yield result
+            # We are interested only in the first infered method,
+            # don't go looking in the rest of the methods of the ancestors.
+            break
 
-nodes.Tuple.infer_binary_op = tl_infer_binary_op
-nodes.List.infer_binary_op = tl_infer_binary_op
-
-
-@bases.yes_if_nothing_inferred
-def instance_infer_binary_op(self, operator, other, context, method):
-    return method.infer_call_result(self, context)
-
-bases.Instance.infer_binary_op = instance_infer_binary_op
+bases.Instance.infer_binary_op = bases.yes_if_nothing_inferred(instance_infer_binary_op)
 
 
 # assignment ##################################################################
@@ -227,7 +243,7 @@ def _resolve_looppart(parts, asspath, context):
                 # search on each possibly inferred value
                 try:
                     for inferred in _resolve_looppart(assigned.infer(context),
-                                                      asspath, context):
+                                                     asspath, context):
                         yield inferred
                 except exceptions.InferenceError:
                     break
@@ -242,7 +258,7 @@ def for_assigned_stmts(self, node, context=None, asspath=None):
                     yield item
     else:
         for inferred in _resolve_looppart(self.iter.infer(context),
-                                          asspath, context):
+                                         asspath, context):
             yield inferred
 
 nodes.For.assigned_stmts = for_assigned_stmts
@@ -282,19 +298,19 @@ def _arguments_infer_argname(self, name, context):
             return
 
     if context and context.callcontext:
-        inferator = arguments.ArgumentInference(context.callcontext)
-        for value in inferator.infer_argument(self.parent, name, context):
+        call_site = arguments.CallSite(context.callcontext)
+        for value in call_site.infer_argument(self.parent, name, context):
             yield value
         return
 
     # TODO: just provide the type here, no need to have an empty Dict.
     if name == self.vararg:
-        vararg = nodes.const_factory(())
+        vararg = node_classes.const_factory(())
         vararg.parent = self
         yield vararg
         return
     if name == self.kwarg:
-        kwarg = nodes.const_factory({})
+        kwarg = node_classes.const_factory({})
         kwarg.parent = self
         yield kwarg
         return
@@ -315,8 +331,8 @@ def arguments_assigned_stmts(self, node, context, asspath=None):
         callcontext = context.callcontext
         context = contextmod.copy_context(context)
         context.callcontext = None
-        return arguments.ArgumentInference(callcontext).infer_argument(
-            self.parent, node.name, context)
+        args = arguments.CallSite(callcontext)
+        return args.infer_argument(self.parent, node.name, context)
     return _arguments_infer_argname(self, node.name, context)
 
 nodes.Arguments.assigned_stmts = arguments_assigned_stmts
@@ -357,7 +373,7 @@ def _resolve_asspart(parts, asspath, context):
                 # possibly inferred value
                 try:
                     for inferred in _resolve_asspart(assigned.infer(context),
-                                                     asspath, context):
+                                                    asspath, context):
                         yield inferred
                 except exceptions.InferenceError:
                     return
@@ -369,92 +385,19 @@ def excepthandler_assigned_stmts(self, node, context=None, asspath=None):
         if isinstance(assigned, nodes.ClassDef):
             assigned = bases.Instance(assigned)
         yield assigned
-
-nodes.ExceptHandler.assigned_stmts = excepthandler_assigned_stmts
-
-
-def _infer_context_manager(self, mgr, context):
-    try:
-        inferred = next(mgr.infer(context=context))
-    except exceptions.InferenceError:
-        return
-    if isinstance(inferred, bases.Generator):
-        # Check if it is decorated with contextlib.contextmanager.
-        func = inferred.parent
-        if not func.decorators:
-            return
-        for decorator_node in func.decorators.nodes:
-            decorator = next(decorator_node.infer(context))
-            if isinstance(decorator, nodes.FunctionDef):
-                if decorator.qname() == _CONTEXTLIB_MGR:
-                    break
-        else:
-            # It doesn't interest us.
-            return
-
-        # Get the first yield point. If it has multiple yields,
-        # then a RuntimeError will be raised.
-        # TODO(cpopa): Handle flows.
-        yield_point = next(func.nodes_of_class(nodes.Yield), None)
-        if yield_point:
-            if not yield_point.value:
-                # TODO(cpopa): an empty yield. Should be wrapped to Const.
-                const = nodes.Const(None)
-                const.parent = yield_point
-                const.lineno = yield_point.lineno
-                yield const
-            else:
-                for inferred in yield_point.value.infer(context=context):
-                    yield inferred
-    elif isinstance(inferred, bases.Instance):
-        try:
-            enter = next(inferred.igetattr('__enter__', context=context))
-        except (exceptions.InferenceError, exceptions.NotFoundError):
-            return
-        if not isinstance(enter, bases.BoundMethod):
-            return
-        for result in enter.infer_call_result(self, context):
-            yield result
+nodes.ExceptHandler.assigned_stmts = bases.raise_if_nothing_inferred(excepthandler_assigned_stmts)
 
 
 @bases.raise_if_nothing_inferred
 def with_assigned_stmts(self, node, context=None, asspath=None):
-    """Infer names and other nodes from a *with* statement.
-
-    This enables only inference for name binding in a *with* statement.
-    For instance, in the following code, inferring `func` will return
-    the `ContextManager` class, not whatever ``__enter__`` returns.
-    We are doing this intentionally, because we consider that the context
-    manager result is whatever __enter__ returns and what it is binded
-    using the ``as`` keyword.
-
-        class ContextManager(object):
-            def __enter__(self):
-                return 42
-        with ContextManager() as f:
-            pass
-        # ContextManager().infer() will return ContextManager
-        # f.infer() will return 42.
-    """
-
-    mgr = next(mgr for (mgr, vars) in self.items if vars == node)
     if asspath is None:
-        for result in _infer_context_manager(self, mgr, context):
-            yield result
-    else:
-        for result in _infer_context_manager(self, mgr, context):
-            # Walk the asspath and get the item at the final index.
-            obj = result
-            for index in asspath:
-                if not hasattr(obj, 'elts'):
-                    raise exceptions.InferenceError
-                try:
-                    obj = obj.elts[index]
-                except IndexError:
-                    util.reraise(exceptions.InferenceError())
-            yield obj
-
-
+        for _, vars in self.items:
+            if vars is None:
+                continue
+            for lst in vars.infer(context):
+                if isinstance(lst, (nodes.Tuple, nodes.List)):
+                    for item in lst.nodes:
+                        yield item
 nodes.With.assigned_stmts = with_assigned_stmts
 
 
@@ -513,22 +456,3 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
                 break
 
 nodes.Starred.assigned_stmts = starred_assigned_stmts
-
-
-def class_as_index(node, context):
-    """Get the value as an index for the given node
-
-    It is expected that the node is an Instance. If it provides
-    an *__index__* method, we'll try to return its int value.
-    """
-    try:
-        for inferred in node.igetattr('__index__', context=context):
-            if not isinstance(inferred, bases.BoundMethod):
-                continue
-
-            for result in inferred.infer_call_result(node, context=context):
-                if (isinstance(result, nodes.Const)
-                        and isinstance(result.value, int)):
-                    return result
-    except exceptions.InferenceError:
-        pass
