@@ -16,8 +16,6 @@
 # You should have received a copy of the GNU Lesser General Public License along
 # with astroid. If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
-
 import unittest
 
 from astroid import bases
@@ -26,24 +24,25 @@ from astroid import nodes
 from astroid import objects
 from astroid import test_utils
 
+
 class ObjectsTest(unittest.TestCase):
 
     def test_frozenset(self):
         node = test_utils.extract_node("""
         frozenset({1: 2, 2: 3}) #@
         """)
-        inferred = next(node.infer())
-        self.assertIsInstance(inferred, objects.FrozenSet)
+        infered = next(node.infer())
+        self.assertIsInstance(infered, objects.FrozenSet)
 
-        self.assertEqual(inferred.pytype(), "%s.frozenset" % bases.BUILTINS)
+        self.assertEqual(infered.pytype(), "%s.frozenset" % bases.BUILTINS)
 
-        itered = inferred.itered()
+        itered = infered.itered()
         self.assertEqual(len(itered), 2)
         self.assertIsInstance(itered[0], nodes.Const)
         self.assertEqual([const.value for const in itered], [1, 2])
 
-        proxied = inferred._proxied
-        self.assertEqual(inferred.qname(), "%s.frozenset" % bases.BUILTINS)
+        proxied = infered._proxied
+        self.assertEqual(infered.qname(), "%s.frozenset" % bases.BUILTINS)
         self.assertIsInstance(proxied, nodes.ClassDef)
 
 
@@ -236,6 +235,7 @@ class SuperTests(unittest.TestCase):
         with self.assertRaises(exceptions.SuperError) as cm:
             first.super_mro()
         self.assertEqual(str(cm.exception), "The first super argument must be type.")
+
         for node in ast_nodes[1:]:
             inferred = next(node.infer())
             self.assertIsInstance(inferred, objects.Super, node)
@@ -251,8 +251,8 @@ class SuperTests(unittest.TestCase):
             def __init__(self):
                 super(Super, self) #@
         ''')
-        inferred = next(node.infer())
-        proxied = inferred._proxied
+        infered = next(node.infer())
+        proxied = infered._proxied
         self.assertEqual(proxied.qname(), "%s.super" % bases.BUILTINS)
         self.assertIsInstance(proxied, nodes.ClassDef)
 
@@ -496,6 +496,17 @@ class SuperTests(unittest.TestCase):
         with self.assertRaises(exceptions.SuperArgumentTypeError):
             inferred.super_mro()
 
+    def test_super_pytype_display_type_name(self):
+        node = test_utils.extract_node('''
+        class A(object):
+            def __init__(self):
+                super(A, self) #@
+        ''')
+        inferred = next(node.infer())
+        self.assertEqual(inferred.pytype(), "%s.super" % bases.BUILTINS)
+        self.assertEqual(inferred.display_type(), 'Super of')
+        self.assertEqual(inferred.name, 'A')
+
 
 if __name__ == '__main__':
     unittest.main()
